from asciimatics.exceptions import NextScene
from asciimatics.widgets import Frame, Layout, Button, Divider, Label


class ConfirmView(Frame):
    def __init__(self, screen, width, height, model):
        super().__init__(
            screen,
            height,
            width,
            can_scroll=False,
            on_load=self._on_load,
            title='Confirmation',
        )
        self._model = model

    def _add_buttons(self):
        layout = Layout([100])
        self.add_layout(layout)
        layout.add_widget(Divider())

        layout = Layout([1, 1])
        self.add_layout(layout)
        layout.add_widget(Button('No', self._back), 0)
        layout.add_widget(Button('Yes', self._next), 1)

    def _add_line(self, text: str = ''):
        layout = Layout([100])
        self.add_layout(layout)
        layout.add_widget(Label(text, align='^'))

    def _add_pair(self, key: str, val: str):
        layout = Layout([10, 1, 10])
        self.add_layout(layout)
        layout.add_widget(Label(key + ':', align='>'), 0)
        layout.add_widget(Label(val, align='<'), 2)

    def _on_load(self):
        for _ in range(2):
            self._add_line()
        for line in self._model.confirm_lines:
            if isinstance(line, str):
                self._add_line(line)
            else:
                # assume tuple
                key, val = line
                self._add_pair(key, val)
        # fill the rest of the space
        self.add_layout(Layout([100], fill_frame=True))

        self._add_buttons()
        self.fix()

    def _back(self):
        raise NextScene(self._model.scene_stack.pop())

    def _next(self):
        self._model.scene_stack.append('Confirm')
        raise NextScene('Transaction')
